package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageInfo;
import com.github.pagehelper.page.PageMethod;
import pps.cloud.base.service.IBasePowerLineCloudService;
import pps.cloud.base.service.IBasePriceStrategyCloudService;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineInput;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineOutput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantInput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantViewOutput;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailInput;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailOutput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerInput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerOutput;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.utils.BaseUtils;
import pps.core.space.entity.*;
import pps.core.space.enums.BusinessError;
import pps.core.space.mapper.*;
import pps.core.space.service.data.SpaceOptimizeDateDuration;
import pps.core.space.service.data.SpaceOptimizeWeight;
import pps.core.space.service.data.SpaceOptimizeWeightDuration;
import pps.core.space.service.data.space_optimize_period.GetSpaceOptimizePeriodInput;
import pps.core.space.service.data.space_optimize_period.GetSpaceOptimizePeriodOutput;
import pps.core.space.service.data.space_optimize_short_period.QuerySpaceOptimizeShortPeriodViewInput;
import pps.core.space.service.data.space_optimize_short_period.QuerySpaceOptimizeShortPeriodViewOutput;
import pps.core.space.service.data.space_optimize_short_wellhead.GetSpaceOptimizeShortWellheadInput;
import pps.core.space.service.data.space_optimize_short_wellhead.GetSpaceOptimizeShortWellheadOutput;
import pps.core.space.service.data.space_optimize_short_wellhead.QuerySpaceOptimizeShortWellheadViewInput;
import pps.core.space.service.data.space_optimize_short_wellhead.QuerySpaceOptimizeShortWellheadViewOutput;
import pps.core.space.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.exception.XServiceException;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiGet;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 极短期间开优化模块
 *
 * @author ZWT
 * @date 2023/09/07
 */
@XService
@XText("极短期间开优化模块")
public class SpaceOptimizeShortPeriodService extends SpaceOptimizeBaseService {

    /**
     * 动态优化
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @XApiGet(anonymous = true)
    @XText("极短期间开优化--动态优化")
    public XServiceResult dynamicOptimize(XContext context) {
        List<SpaceOptimizePeriodDTO> periodDTOList = new ArrayList<>(32);
        List<SpaceOptimizeWellheadDTO> wellheadDTOList = new ArrayList<>(64);
        List<SpaceOptimizeDurationDTO> durationDTOList = new ArrayList<>(128);
        List<SpaceOptimizeDurationDTO> unOptimizeDurationList = new ArrayList<>(128);
        //取当前时间
        DateTime startDate = DateUtil.beginOfDay(DateTime.now());
        //取生效中的基础间开
        List<SpaceInstitutionDetailEnt> detailEntList = this.getEffectiveSpaceInstitution(context, startDate.toString());
        //取线路ID和制度ID
        List<String> lineIds = new ArrayList<>(32);
        List<String> institutionIds = new ArrayList<>(32);
        detailEntList.forEach(detail -> {
            lineIds.add(detail.getLineId());
            institutionIds.add(detail.getId());
        });
        //取间开的所有井口
        SpaceInstitutionWellheadViewMapper wellheadMapper = context.getBean(SpaceInstitutionWellheadViewMapper.class);
        List<SpaceInstitutionWellheadView> spaceWellheadList = wellheadMapper.selectWellheadsByInstitutionIds(institutionIds);
        if (CollUtil.isEmpty(spaceWellheadList)) {
            throw new XServiceException(BusinessError.DidNotFindWellhead);
        }
        //取间开对应的线路
        IBasePowerLineCloudService lineCloudService = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineOutput> lineResult = lineCloudService.getBasePowerLineList(context, DynamicQueryBasePowerLineInput.builder()
                .lineIds(lineIds)
                .build());
        lineResult.throwIfFail();
        //查线路下所有电站
        XListResult<DynamicQueryBasePowerLinePlantViewOutput> linePlantResult = lineCloudService.getPowerLinePlantViewList(context, DynamicQueryBasePowerLinePlantInput.builder()
                .lineIds(lineIds)
                .build());
        linePlantResult.throwIfFail();
        //取优化截至日期
        DateTime optimizeDate = DateUtil.offsetDay(startDate, BusinessConstant.ONE);
        //取当前月
        int month = startDate.monthBaseOne();
        //查市电峰谷
        IBasePriceStrategyCloudService strategyCloudService = context.getBean(IBasePriceStrategyCloudService.class);
        XListResult<GetBasePriceStrategyDetailOutput> strategyResult = strategyCloudService.queryStrategyDetailList(context, GetBasePriceStrategyDetailInput.builder()
                .strategyMonth(String.valueOf(month))
                .build());
        strategyResult.throwIfFail();
        //查井口配置时间段
        SpaceInstitutionDurationMapper durationMapper = context.getBean(SpaceInstitutionDurationMapper.class);
        List<SpaceInstitutionDurationEnt> durationList = durationMapper.selectList(new QueryWrapper<SpaceInstitutionDurationEnt>()
                .select("*",
                        "STR_TO_DATE( CONCAT( open_well_time, ':00' ), '%H:%i:%s' ) AS openTime",
                        "STR_TO_DATE( CONCAT( close_well_time, ':00' ), '%H:%i:%s' ) AS closeTime"
                )
                .lambda()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .in(SpaceInstitutionDurationEnt::getWellheadId, spaceWellheadList.stream().map(SpaceInstitutionWellheadView::getWellheadId).collect(Collectors.toList()))
                .orderByAsc(SpaceInstitutionDurationEnt::getOpenWellTime)
        );
        if (CollUtil.isEmpty(durationList)) {
            throw new XServiceException(BusinessError.DidNotFindCalibration);
        }
        //间开制度ID/井口LIST Map
        Map<String, List<SpaceInstitutionWellheadView>> wellheadMap = spaceWellheadList.stream()
                .collect(Collectors.groupingBy(SpaceInstitutionWellheadView::getInstitutionId));
        //线路ID/峰谷策略ID Map
        Map<String, String> lineMap = lineResult.getResult().stream()
                .collect(Collectors.toMap(DynamicQueryBasePowerLineOutput::getId, DynamicQueryBasePowerLineOutput::getStrategyId));
        //线路ID/电站ID LIST Map
        Map<String, List<String>> plantIdsMap = linePlantResult.getResult().stream()
                .collect(Collectors.groupingBy(DynamicQueryBasePowerLinePlantViewOutput::getLineId,
                        Collectors.mapping(DynamicQueryBasePowerLinePlantViewOutput::getPlantId, Collectors.toList())));
        //峰谷策略ID/峰谷策略LIST Map
        Map<String, List<GetBasePriceStrategyDetailOutput>> strategyMap = strategyResult.getResult().stream()
                .collect(Collectors.groupingBy(GetBasePriceStrategyDetailOutput::getStrategyId));
        //间开制度ID/区间配置Map Map
        Map<String, Map<String, List<SpaceInstitutionDurationEnt>>> durationMap = durationList.stream()
                .collect(Collectors.groupingBy(SpaceInstitutionDurationEnt::getInstitutionId,
                        Collectors.groupingBy(SpaceInstitutionDurationEnt::getWellheadId)));
        //初始化时间轴
        List<DateTime> rangeToList = DateUtil.rangeToList(DateUtil.offsetDay(BusinessConstant.DATE_FLAG, -1), BusinessConstant.DATE_FLAG, DateField.MINUTE, 30);
        //遍历
        List<DynamicQueryPlantPredictedPowerOutput> powerList;
        List<GetBasePriceStrategyDetailOutput> strategyList;
        List<SpaceInstitutionWellheadView> wellheadList;
        String periodId;
        //权重数组
        int[] ladder = {10, 5, 3, 1};
        for (SpaceInstitutionDetailEnt detail : detailEntList) {
            //创建记录
            periodId = this.createOptimizePeriod(periodDTOList, detail.getId(), detail.getLineId(), BaseUtils.getExecutionCycleForCalibration(startDate, optimizeDate), optimizeDate);
            //查预测光伏出力
            powerList = this.getAveragePowerHalfHour(context, DynamicQueryPlantPredictedPowerInput.builder()
                    .plantIds(plantIdsMap.get(detail.getLineId()))
                    .startTime(startDate.toString(BusinessConstant.DATE_FORMAT_DAY))
                    .endTime(optimizeDate.toString(BusinessConstant.DATE_FORMAT_DAY))
                    .dateType(BusinessConstant.ONE)
                    .build());
            if (CollUtil.isEmpty(powerList)) {
                //没有预测数据都不优化
                this.setUnOptimizeWellheadConfig(durationDTOList, unOptimizeDurationList, wellheadDTOList,
                        spaceWellheadList, durationMap.get(detail.getId()), periodId, optimizeDate);
                continue;
            }
            //取峰谷策略
            strategyList = strategyMap.get(lineMap.get(detail.getLineId()));
            //取井口并遍历
            wellheadList = wellheadMap.get(detail.getId());
            BigDecimal serviceRating = BigDecimal.ZERO;
            SpaceInstitutionWellheadView wellhead;
            for (int i = 0; i < wellheadList.size(); i++) {
                wellhead = wellheadList.get(i);
                serviceRating = serviceRating.add(wellhead.getServiceRating());
                //计算权重
                int rangeListSize = rangeToList.size();
                int rangeIndex = 0;
                int powerListSize = powerList.size();
                int powerIndex = 0;
                int strategyListSize = strategyList.size();
                int strategyIndex = 0;
                List<SpaceOptimizeWeight> weightList = new ArrayList<>(rangeListSize);
                DynamicQueryPlantPredictedPowerOutput firstPower;
                DynamicQueryPlantPredictedPowerOutput lastPower;
                GetBasePriceStrategyDetailOutput firstStrategy;
                GetBasePriceStrategyDetailOutput lastStrategy;
                while (rangeListSize > 0 && rangeListSize > rangeIndex) {
                    boolean powerFlag = false;
                    SpaceOptimizeWeight firstWeight = new SpaceOptimizeWeight();
                    DateTime firstTime = rangeToList.get(rangeIndex);
                    firstWeight.setTimestamp(firstTime);
                    firstWeight.setSort(rangeIndex);
                    firstPower = powerList.get(powerIndex);
                    if (0 == firstTime.compareTo(firstPower.getCreateTime())) {
                        firstWeight.setPower(firstPower.getPower());
                        //判断发电量是否满足运行功率
                        powerFlag = firstPower.getPower().compareTo(serviceRating) >= 0;
                        if (powerFlag) {
                            firstWeight.setWeight(10);
                        }
                        powerIndex++;
                    }
                    firstStrategy = strategyList.get(strategyIndex);
                    Date closeTime = ObjectUtil.isEmpty(firstStrategy.getCloseTime()) && CharSequenceUtil.equals(BusinessConstant.END_OF_DAY_TIME, firstStrategy.getStartTime()) ? BusinessConstant.DATE_FLAG : firstStrategy.getCloseTime();
                    //在市电峰谷时段内,且未满足运行功率,设置权重
                    if (DateUtil.isIn(firstTime, firstStrategy.getOpenTime(), closeTime)) {
                        if (CharSequenceUtil.equals(firstStrategy.getPeriodTypeKey(), "RUSH")) {
                            //尖峰时段要停井
                            firstWeight.setWeight(0);
                        } else if (!powerFlag) {
                            firstWeight.setWeight(this.getWeightByPeriodTypeKey(firstStrategy.getPeriodTypeKey()));
                        }
                    }
                    if (firstTime.compareTo(firstStrategy.getCloseTime()) >= 0) {
                        strategyIndex++;
                    }
                    weightList.add(firstWeight);
                    rangeIndex++;
                    rangeListSize--;
                    //防止重复插入
                    if (rangeIndex >= rangeListSize) {
                        continue;
                    }
                    /*----------------------------------------------------*/
                    powerFlag = false;
                    SpaceOptimizeWeight lastWeight = new SpaceOptimizeWeight();
                    DateTime lastTime = rangeToList.get(rangeListSize);
                    lastWeight.setTimestamp(lastTime);
                    lastWeight.setSort(rangeListSize);
                    lastPower = powerList.get(powerListSize - 1);
                    if (0 == lastTime.compareTo(lastPower.getCreateTime())) {
                        lastWeight.setPower(lastPower.getPower());
                        //判断发电量是否满足运行功率
                        powerFlag = lastPower.getPower().compareTo(serviceRating) >= 0;
                        if (powerFlag) {
                            lastWeight.setWeight(10);
                        }
                        powerListSize--;
                    }
                    lastStrategy = strategyList.get(strategyListSize - 1);
                    closeTime = ObjectUtil.isEmpty(lastStrategy.getCloseTime()) && CharSequenceUtil.equals(BusinessConstant.END_OF_DAY_TIME, lastStrategy.getEndTime()) ? BusinessConstant.DATE_FLAG : lastStrategy.getCloseTime();
                    //在市电峰谷时段内,且未满足运行功率,设置权重
                    if (DateUtil.isIn(lastTime, lastStrategy.getOpenTime(), closeTime)) {
                        if (CharSequenceUtil.equals(lastStrategy.getPeriodTypeKey(), "RUSH")) {
                            //尖峰时段要停井
                            lastWeight.setWeight(0);
                        } else if (!powerFlag) {
                            lastWeight.setWeight(this.getWeightByPeriodTypeKey(lastStrategy.getPeriodTypeKey()));
                        }
                    }
                    if (lastTime.compareTo(lastStrategy.getOpenTime()) <= 0) {
                        strategyListSize--;
                    }
                    weightList.add(lastWeight);
                }
                //排序
                weightList = weightList.stream()
                        .sorted(Comparator.comparing(SpaceOptimizeWeight::getSort))
                        .collect(Collectors.toList());
                SpaceOptimizeWeight weight;
                //测试用 ====== start
                int dayOpen = 15;
                int maxOpen = 5;
                int minOpen = 3;
                int maxClose = 2;
                int minClose = 1;
                int minOpenMinute = minOpen * 60;
                int maxOpenMinute = maxOpen * 60;
                int minCloseMinute = minClose * 60;
                int maxCloseMinute = maxClose * 60;
                int sumMinute = minCloseMinute + maxOpenMinute;
                //测试用 ====== end
                //分级取时间段
                List<SpaceOptimizeWeightDuration> weightDurationList = new ArrayList<>(32);
                long between;
                for (int i1 = 0; i1 < ladder.length; i1++) {
                    int begin = -1;
                    //取每级时间
                    for (int i2 = 0; i2 < weightList.size(); i2++) {
                        weight = weightList.get(i2);
                        //过滤条件:权重相同
                        if (ladder[i1] == weight.getWeight()) {
                            //确定开始时间位置
                            if (begin == -1) {
                                begin = i2;
                            }
                        } else if (begin != -1) {
                            //如果开井时间不满足最小开井时间则舍弃
                            between = DateUtil.between(weightList.get(begin).getTimestamp(), weightList.get(i2).getTimestamp(), DateUnit.MINUTE);
                            if (minOpenMinute <= between) {
                                //创建区间
                                weightDurationList.add(
                                        SpaceOptimizeWeightDuration.builder()
                                                .openTime(weightList.get(begin).getTimestamp())
                                                .closeTime(weightList.get(i2).getTimestamp())
                                                .openIndex(begin)
                                                .closeIndex(i2)
                                                .duration(between)
                                                .weight(ladder[i1])
                                                .build()
                                );
                            }
                            begin = -1;
                        }
                    }
                }
                weightDurationList = weightDurationList.stream()
                        .sorted(Comparator.comparing(SpaceOptimizeWeightDuration::getOpenTime))
                        .collect(Collectors.toList());
                //计算第一次启动时间
                int startIndex = 0;
                int weightNum = 0;
                for (int i1 = 0; i1 < weightDurationList.size(); i1++) {
                    if (weightDurationList.get(i1).getWeight() > weightNum) {
                        weightNum = weightDurationList.get(i1).getWeight();
                        startIndex = i1;
                    }
                }
                List<SpaceOptimizeDurationDTO> optimizeDurationDTOList = new ArrayList<>(32);
                //从前往后算时间
                SpaceOptimizeWeightDuration thisWeightDuration;
                SpaceOptimizeWeightDuration nextWeightDuration;
                long duration;
                DateTime openTime;
                for (int i1 = startIndex; i1 < weightDurationList.size(); i1++) {
                    thisWeightDuration = weightDurationList.get(i1);
                    duration = thisWeightDuration.getDuration();
                    openTime = thisWeightDuration.getOpenTime();
                    //判断时间间隔是否能满足最大开井时间
                    if (duration >= maxOpenMinute) {
                        //满足,判断能满足几次(最大开井时间)
                        long count = duration / maxOpenMinute;
                        if (count > 0) {
                            //todo : 能力有限,只能用(最大开井时间+最小停井时间)固定范围,求不了最优排布
                            for (int i2 = 1; i2 <= count; i2++) {
                                DateTime closeTime;
                                if (duration >= maxOpenMinute) {
                                    closeTime = openTime.offsetNew(DateField.MINUTE, maxOpenMinute);
                                    duration = duration - maxOpenMinute - minCloseMinute;
                                } else if (duration >= minOpenMinute) {
                                    closeTime = openTime.offsetNew(DateField.MINUTE, minOpenMinute);
                                    duration = duration - minOpenMinute - minCloseMinute;
                                } else {
                                    //时间不够,舍弃
                                    break;
                                }
                                optimizeDurationDTOList.add(SpaceOptimizeDurationDTO.builder()
                                        .openTime(openTime)
                                        .closeTime(closeTime)
                                        .build());
                                openTime = closeTime;
                            }
                        } else {
                            //不满足,取全部
                            optimizeDurationDTOList.add(SpaceOptimizeDurationDTO.builder()
                                    .openTime(openTime)
                                    .closeTime(thisWeightDuration.getCloseTime())
                                    .build());
                            duration = 0;
                        }
                        //duration小于0:关井中,大于等于0,可以开井
                        if (duration < 0) {

                        }

                        System.out.println();

                    } else {
                        //不满足,使用最小开井时间
                    }

                }
                System.out.println();
            }
        }
        return XServiceResult.OK;
    }

    /**
     * 通过时段类型获取权重
     *
     * @param periodTypeKey 周期类型键
     * @return int
     */
    private int getWeightByPeriodTypeKey(String periodTypeKey) {
        switch (periodTypeKey) {
            case "LOW":
                return 5;
            case "PEACE":
                return 3;
            case "PEAK":
                return 1;
            case "RUSH":
            default:
                return 0;
        }
    }

    /**
     * 极短期间开开优化--分页列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QuerySpaceOptimizeShortPeriodViewOutput}>
     */
    @XApiGet
    @XText("极短期间开优化--分页列表")
    public XPageResult<QuerySpaceOptimizeShortPeriodViewOutput> querySpaceOptimizeShortPeriodView(XContext context, QuerySpaceOptimizeShortPeriodViewInput input) {
        SpaceOptimizeShortPeriodViewMapper mapper = context.getBean(SpaceOptimizeShortPeriodViewMapper.class);
        SpaceOptimizeShortPeriodView record = XCopyUtils.copyNewObject(input, SpaceOptimizeShortPeriodView.class);
        PageMethod.startPage(input.getPage(), input.getLimit());
        List<SpaceOptimizeShortPeriodView> list = mapper.selectList(record);
        PageInfo<SpaceOptimizeShortPeriodView> pageInfo = new PageInfo<>(list);
        List<QuerySpaceOptimizeShortPeriodViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySpaceOptimizeShortPeriodViewOutput.class);
        if (CollUtil.isNotEmpty(outputs)) {
            Map<String, String> gridTypeMap = ServiceUtil.getDictMap(context, BusinessConstant.GRID_TYPE);
            Map<String, String> ouMap = ServiceUtil.getOuMapByOuIds(context, outputs.stream()
                    .map(QuerySpaceOptimizeShortPeriodViewOutput::getOuId)
                    .distinct()
                    .collect(Collectors.toList())
            );
            String lineName = ServiceUtil.getPowerLineNameById(context, input.getLineId());
            outputs.forEach(output -> {
                output.setLineName(lineName);
                output.setOuName(ouMap.get(output.getOuId()));
                output.setGridTypeName(gridTypeMap.get(output.getGridTypeKey()));
                output.setOptimizeStateValue(output.getOptimizeState().equals(0) ? BusinessConstant.OPTIMIZED_C : BusinessConstant.NO_OPTIMIZATION_C);
            });
        }
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /**
     * 极短期间开开优化--详情
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetSpaceOptimizePeriodOutput}>
     */
    @XApiGet
    @XText("极短期间开优化--详情")
    public XSingleResult<GetSpaceOptimizePeriodOutput> getSpaceOptimizeShortPeriodView(XContext context, GetSpaceOptimizePeriodInput input) {
        return XSingleResult.success(super.getSpaceOptimizePeriodView(context, input, BusinessConstant.FOUR, BusinessConstant.ONE));
    }

    /**
     * 获取当日井口优化列表
     * GET /space/space-optimize-short-period/get-optimize-list
     * 接口ID：158457157
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-158457157
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link GetSpaceOptimizeShortWellheadOutput}>
     */
    @XApiGet
    @XText("极短期间开优化--获取当日井口优化列表")
    public XListResult<GetSpaceOptimizeShortWellheadOutput> getOptimizeList(XContext context, GetSpaceOptimizeShortWellheadInput input) {
        DateTime now = DateUtil.beginOfDay(DateTime.now());
        String lineId = input.getLineId();
        String wellheadId = input.getWellheadId();
        SpaceOptimizeShortPeriodMapper mapper = context.getBean(SpaceOptimizeShortPeriodMapper.class);
        SpaceOptimizeShortPeriodEnt periodEnt = mapper.selectOne(new LambdaQueryWrapper<SpaceOptimizeShortPeriodEnt>()
                .eq(SpaceOptimizeShortPeriodEnt::getOptimizeDate, now)
                .eq(SpaceOptimizeShortPeriodEnt::getLineId, lineId)
                .last("limit 1")
        );
        if (ObjectUtil.isEmpty(periodEnt)) {
            return XListResult.success(Collections.emptyList());
        }
        SpaceOptimizeShortWellheadMapper wellheadMapper = context.getBean(SpaceOptimizeShortWellheadMapper.class);
        List<SpaceOptimizeShortWellheadEnt> wellheadList = wellheadMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeShortWellheadEnt>()
                .eq(SpaceOptimizeShortWellheadEnt::getShortPeriodId, periodEnt.getId())
                .eq(CharSequenceUtil.isNotBlank(wellheadId), SpaceOptimizeShortWellheadEnt::getWellheadId, wellheadId)
        );
        if (CollUtil.isEmpty(wellheadList)) {
            return XListResult.success(Collections.emptyList());
        }
        List<GetSpaceOptimizeShortWellheadOutput> wellheadOutputs = XCopyUtils.copyNewList(wellheadList, GetSpaceOptimizeShortWellheadOutput.class);
        SpaceOptimizeShortDurationMapper durationMapper = context.getBean(SpaceOptimizeShortDurationMapper.class);
        List<SpaceOptimizeShortDurationEnt> durationList = durationMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeShortDurationEnt>()
                .eq(SpaceOptimizeShortDurationEnt::getShortPeriodId, periodEnt.getId())
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .eq(SpaceOptimizeShortDurationEnt::getIsOptimize, BusinessConstant.ZERO)
                .eq(CharSequenceUtil.isNotBlank(wellheadId), SpaceOptimizeShortDurationEnt::getWellheadId, wellheadId)
                .orderByAsc(SpaceOptimizeShortDurationEnt::getIsOptimize, SpaceOptimizeShortDurationEnt::getOpenWellTime)
        );
        List<SpaceOptimizeDateDuration> durationOutputs = XCopyUtils.copyNewList(durationList, SpaceOptimizeDateDuration.class);
        Map<String, List<SpaceOptimizeDateDuration>> collect = durationOutputs.stream()
                .collect(Collectors.groupingBy(SpaceOptimizeDateDuration::getWellheadId));
        for (GetSpaceOptimizeShortWellheadOutput wellhead : wellheadOutputs) {
            wellhead.setOptimizeList(ServiceUtil.getAllDateDuration(collect.get(wellhead.getWellheadId())));
        }
        return XListResult.success(wellheadOutputs);
    }

    /**
     * 先导实验井间开制度
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QuerySpaceOptimizeShortWellheadViewOutput}>
     */
    @XApiGet
    @XText("极短期间开优化--先导实验井间开制度")
    public XPageResult<QuerySpaceOptimizeShortWellheadViewOutput> getDailySystemDisplay(XContext context, QuerySpaceOptimizeShortWellheadViewInput input) {
        SpaceOptimizeShortDurationViewMapper mapper = context.getBean(SpaceOptimizeShortDurationViewMapper.class);
        PageMethod.startPage(input.getPage(), input.getLimit());
        List<SpaceOptimizeShortDurationView> list = mapper.selectDailySystemDisplay(SpaceOptimizeShortDurationView.builder()
                .queryDate(DateUtil.beginOfDay(DateUtil.date()))
                .ouId(input.getStationId())
                .build());
        PageInfo<SpaceOptimizeShortDurationView> pageInfo = new PageInfo<>(list);
        return XPageResult.success(XCopyUtils.copyNewList(pageInfo.getList(), QuerySpaceOptimizeShortWellheadViewOutput.class),
                input, pageInfo.getTotal());
    }
}